@@ -30,13 +30,31 @@
 
 #include <GNUstepGUI/GSFontInfo.h>
 #include <cairo.h>
+#define id fontconfig_id
 #include <fontconfig/fontconfig.h>
+#undef id
 #include "cairo/CairoFaceInfo.h"
 
 @interface CairoFontEnumerator : GSFontEnumerator
 + (CairoFaceInfo *) fontWithName: (NSString *)name;
 @end
 
+@interface FontconfigPatternGenerator : NSObject
+{
+  NSDictionary *_attributes;
+  FcPattern *_pat;
+}
+- (FcPattern *)createPatternWithAttributes: (NSDictionary *)attributes;
+@end
+
+@interface FontconfigPatternParser : NSObject
+{
+  NSMutableDictionary *_attributes;
+  FcPattern *_pat;
+}
+- (NSDictionary*)attributesFromPattern: (FcPattern *)pat;
+@end
+
 @interface FontconfigCharacterSet : NSCharacterSet
 {
   FcCharSet *_charset;
