@@ -178,14 +178,22 @@ typedef struct w32serverFlags {
 - (void) decodeWM_QUERYOPENParams: (WPARAM)wParam : (LPARAM)lParam : (HWND)hwnd;
 - (void) decodeWM_SYSCOMMANDParams: (WPARAM)wParam : (LPARAM)lParam : (HWND)hwnd;
 - (void) decodeWM_COMMANDParams: (WPARAM)wParam : (LPARAM)lParam : (HWND)hwnd;
+- (void) decodeWM_THEMECHANGEDParams: (WPARAM)wParam : (LPARAM)lParam : (HWND)hwnd;
 
 @end
 
-typedef struct _win_intern {
-  int32_t level;
+// FIXME: Keep all of the extra window data in one place
+
+// Extra window data accessed via GetWindowLong
+
 #define OFF_LEVEL	0
-  int32_t orderedIn;
-#define OFF_ORDERED	sizeof(int32_t)
+#define OFF_ORDERED	sizeof(DWORD)
+#define WIN_EXTRABYTES (2*sizeof(DWORD))
+
+// Extra window data allocated using objc_malloc in WM_CREATE and accessed via
+// the GWL_USERDATA pointer
+
+typedef struct _win_intern {
   BOOL useHDC;
   BOOL backingStoreEmpty;
   HDC hdc; 
