@@ -356,8 +356,9 @@ draw_info_t ART_DI;
 }
 
 
-- (void) GSShowGlyphs: (const NSGlyph *)glyphs : (size_t) length
+- (void) GSShowGlyphsWithAdvances: (const NSGlyph *)glyphs : (const NSSize *)advances : (size_t) length
 {
+  // FIXME: Currently advances is ignored
   NSPoint p;
   int x, y;
 
