@@ -73,28 +73,31 @@
     NSZoneFree(NSDefaultMallocZone(), _base); \
   }
 
-static float floatFromUserSpace(NSAffineTransform *ctm, float f)
+static inline double doubleFromUserSpace(NSAffineTransform *ctm, double d)
 {
-  NSSize s = {f, f};
-
   if (ctm)
     {
+      NSSize s = {d, d};
+
       s = [ctm transformSize: s];
-      f = (((s.width > 0.0) ? s.width : -s.width) + 
+      d = (((s.width > 0.0) ? s.width : -s.width) + 
            ((s.height > 0.0) ? s.height : -s.height)) / 2;
     }
-  return f;
+  return d;
 }
 
-static float floatToUserSpace(NSAffineTransform *ctm, float f)
+static inline float floatToUserSpace(NSAffineTransform *ctm, double d)
 {
-  NSAffineTransform *ictm;
-  
-  ictm = [ctm copyWithZone: [ctm zone]];
-  [ictm invert];
-  f = floatFromUserSpace(ictm, f);
-  RELEASE(ictm);
-  return f;
+  if (ctm)
+    {
+      NSAffineTransform *ictm;
+      
+      ictm = [ctm copy];
+      [ictm invert];
+      d = doubleFromUserSpace(ictm, d);
+      RELEASE(ictm);
+    }
+  return (float)d;
 }
 
 
@@ -229,8 +232,6 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
                       cairo_clip(copy->_ct);
                     }
                 }
-
-              cairo_rectangle_list_destroy(clip_rects);
             }
           else if (status == CAIRO_STATUS_CLIP_NOT_REPRESENTABLE)
             {
@@ -248,6 +249,8 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
             {
               NSLog(@"Cairo status '%s' in copy clip", cairo_status_to_string(status));
             }
+
+          cairo_rectangle_list_destroy(clip_rects);
 #endif
         }
     }
@@ -312,7 +315,7 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
   NSPoint p;
 
   p = [path currentPoint];
-  cairo_move_to(_ct, floorf(p.x), floorf(p.y));
+  cairo_move_to(_ct, p.x, p.y);
 }
 
 - (void) DPScharpath: (const char *)s : (int)b
@@ -408,8 +411,7 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
 {
   if (_ct)
     {
-      size = floatFromUserSpace(ctm, size);
-      cairo_set_font_size(_ct, size);
+      cairo_set_font_size(_ct, doubleFromUserSpace(ctm, size));
     }
 }
 
@@ -417,8 +419,8 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
 {
   if (_ct)
     {
-      GS_BEGINITEMBUF(chars, length + 1, char);
       device_color_t c;
+      GS_BEGINITEMBUF(chars, length + 1, char);
 
       c = strokeColor;
       gsColorToRGB(&c);
@@ -433,8 +435,10 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
     }
 }
 
-- (void) GSShowGlyphs: (const NSGlyph *)glyphs : (size_t)length
+- (void) GSShowGlyphsWithAdvances: (const NSGlyph *)glyphs : (const NSSize *)advances : (size_t) length
 {
+  // FIXME: this method should just be a call to cairo_show_glyphs
+  // FIXME: Currently advances is ignored
   if (_ct)
     {
       cairo_matrix_t local_matrix;
@@ -458,9 +462,18 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
       cairo_transform(_ct, &local_matrix);
 
       [(CairoFontInfo *)font drawGlyphs: glyphs
-                        length: length
-                        on: _ct];
+				 length: length
+				     on: _ct];
+
+      double x, y;
+      cairo_get_current_point(_ct, &x, &y);
+      cairo_user_to_device(_ct, &x, &y);
+      
       cairo_restore(_ct);
+      
+      // reset the current point
+      cairo_device_to_user(_ct, &x, &y);
+      [path moveToPoint: NSMakePoint(x,y)];
     }
 }
 
@@ -504,8 +517,6 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
   cairo_set_line_width(_ct, 1.0);
   cairo_set_operator(_ct, CAIRO_OPERATOR_OVER);
   cairo_new_path(_ct);
-
-  _strokeadjust = 1;
 }
 
 - (void) DPScurrentflat: (float *)flatness
@@ -576,8 +587,7 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
 {
   if (_ct)
     {
-      *width = (float)cairo_get_line_width(_ct);
-      *width = floatToUserSpace(ctm, *width);
+      *width = floatToUserSpace(ctm, cairo_get_line_width(_ct));
     }
 }
 
@@ -585,30 +595,28 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
 {
   if (_ct)
     {
-      *limit = (float)cairo_get_miter_limit(_ct);
-      *limit = floatToUserSpace(ctm, *limit);
+      *limit = floatToUserSpace(ctm, cairo_get_miter_limit(_ct));
     }
 }
 
 - (void) DPScurrentstrokeadjust: (int *)b
 {
-    // FIXME
 }
 
 - (void) DPSsetdash: (const float *)pat : (int)size : (float)foffset
 {
   if (_ct)
     {
-      GS_BEGINITEMBUF(dpat, size, double);
-      double doffset = (double)floatFromUserSpace(ctm, foffset);
+      double doffset = doubleFromUserSpace(ctm, foffset);
       int i;
+      GS_BEGINITEMBUF(dpat, size, double);
 
       i = size;
       while (i)
         {
           i--;
           // FIXME: When using the correct values, some dashes look wrong
-          dpat[i] = (double)floatFromUserSpace(ctm, pat[i]) * 1.4;
+          dpat[i] = doubleFromUserSpace(ctm, pat[i]) * 1.4;
         }
       cairo_set_dash(_ct, dpat, size, doffset);
       GS_ENDITEMBUF();
@@ -646,7 +654,7 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
 {
   if (_ct)
     {
-      cairo_set_line_width(_ct, floatFromUserSpace(ctm, width));
+      cairo_set_line_width(_ct, doubleFromUserSpace(ctm, width));
     }
 }
 
@@ -654,109 +662,25 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
 {
   if (_ct)
     {
-      cairo_set_miter_limit(_ct, floatFromUserSpace(ctm, limit));
+      cairo_set_miter_limit(_ct, doubleFromUserSpace(ctm, limit));
     }
 }
 
 - (void) DPSsetstrokeadjust: (int)b
 {
-  _strokeadjust = b;
 }
 
 /*
  * Path operations
  */
 
-- (void) _adjustPath: (float)offs
-{
-  unsigned            count = [path elementCount];
-  NSBezierPathElement type;
-  NSPoint             points[3] = {{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}};
-  NSPoint             last_points[3] = {{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}};
-  unsigned            i;
-  int                 index, last_index;
-
-  for (i = 0; i < count; i++)
-    {
-      type = [path elementAtIndex:i associatedPoints:points];
-
-      if (type == NSCurveToBezierPathElement) break;
-
-      points[0].x = floorf(points[0].x) + offs;
-      points[0].y = floorf(points[0].y) + offs;
-
-      index = i;
-      last_index = i - 1;
-
-      if (type == NSClosePathBezierPathElement)
-        {
-          index = 0;
-          [path elementAtIndex:0 associatedPoints:points];
-          if (fabs(last_points[0].x - points[0].x) < 1.0)
-            {
-              last_points[0].x = floorf(last_points[0].x) + offs;
-              points[0].x = last_points[0].x;
-            }
-          else if (fabs(last_points[0].y - points[0].y) < 1.0)
-            {
-              last_points[0].y = floorf(last_points[0].y) + offs;
-              points[0].y = last_points[0].y;
-            }
-          else
-            {
-              index = -1;
-            }
-        }
-      else if (fabs(last_points[0].x - points[0].x) < 1.0)
-        { // Vertical path
-          points[0].x = floorf(points[0].x) + offs;
-          points[0].y = floorf(points[0].y);
-          if (type == NSLineToBezierPathElement)
-            {
-              last_points[0].x = points[0].x;
-            }
-        }
-      else if (fabs(last_points[0].y - points[0].y) < 1.0)
-        { // Horizontal path
-          points[0].x = floorf(points[0].x);
-          points[0].y = floorf(points[0].y) + offs;
-          if (type == NSLineToBezierPathElement)
-            {
-              last_points[0].y = points[0].y;
-            }
-        }
-
-      // Save adjusted values into NSBezierPath
-      if (index >= 0)
-        [path setAssociatedPoints:points atIndex:index];
-      if (last_index >= 0)
-        [path setAssociatedPoints:last_points atIndex:last_index];
-
-      last_points[0].x = points[0].x;
-      last_points[0].y = points[0].y;
-    }
-}
-
-- (void) _setPath: (BOOL)fillOrClip
+- (void) _setPath
 {
   unsigned count = [path elementCount];
   unsigned i;
   SEL elmsel = @selector(elementAtIndex:associatedPoints:);
   IMP elmidx = [path methodForSelector: elmsel];
 
-  if (_strokeadjust)
-    {
-      float offs;
-
-      if ((remainderf((float)cairo_get_line_width(_ct), 2.0) == 0.0)
-          || fillOrClip == YES)
-        offs = 0.0;
-      else
-        offs = 0.5;
-
-      [self _adjustPath:offs];
-    }
-
   // reset current cairo path
   cairo_new_path(_ct);
   for (i = 0; i < count; i++) 
@@ -791,7 +715,7 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
 {
   if (_ct)
     {
-      [self _setPath:YES];
+      [self _setPath];
       cairo_clip(_ct);
      }
 }
@@ -800,7 +724,7 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
 {
   if (_ct)
     {
-      [self _setPath:YES];
+      [self _setPath];
       cairo_set_fill_rule(_ct, CAIRO_FILL_RULE_EVEN_ODD);
       cairo_clip(_ct);
       cairo_set_fill_rule(_ct, CAIRO_FILL_RULE_WINDING);
@@ -823,7 +747,7 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
       gsColorToRGB(&c);
       // The underlying concept does not allow to determine if alpha is set or not.
       cairo_set_source_rgba(_ct, c.field[0], c.field[1], c.field[2], c.field[AINDEX]);
-      [self _setPath:YES];
+      [self _setPath];
       cairo_set_fill_rule(_ct, CAIRO_FILL_RULE_EVEN_ODD);
       cairo_fill(_ct);
       cairo_set_fill_rule(_ct, CAIRO_FILL_RULE_WINDING);
@@ -847,7 +771,7 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
       gsColorToRGB(&c);
       // The underlying concept does not allow to determine if alpha is set or not.
       cairo_set_source_rgba(_ct, c.field[0], c.field[1], c.field[2], c.field[AINDEX]);
-      [self _setPath:YES];
+      [self _setPath];
       cairo_fill(_ct);
     }
   [self DPSnewpath];
@@ -865,14 +789,33 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
 {
   if (_ct)
     {
+      BOOL zeroLineWidth = NO;
       device_color_t c;
 
       c = strokeColor;
       gsColorToRGB(&c);
       // The underlying concept does not allow to determine if alpha is set or not.
       cairo_set_source_rgba(_ct, c.field[0], c.field[1], c.field[2], c.field[AINDEX]);
-      [self _setPath:NO];
+      [self _setPath];
+
+      // If the line width is 0, draw a thin line.
+      // cairo (and Quartz) will draw nothing with a line width of 0.
+      // See the PostScript Language Reference, 3rd ed, p. 674
+      if (cairo_get_line_width(_ct) == 0)
+	{
+	  double w1 = 1.0;
+	  double w2 = 1.0;
+	  cairo_device_to_user_distance(_ct, &w1, &w2);
+	  cairo_set_line_width(_ct, w1);
+	  zeroLineWidth = YES;
+	}
+
       cairo_stroke(_ct);
+
+      if (zeroLineWidth)
+	{
+	  cairo_set_line_width(_ct, 0);
+	}
     }
   [self DPSnewpath];
 }
@@ -1254,7 +1197,7 @@ _set_op(cairo_t *ct, NSCompositingOperation op)
       // This is almost a rectclip::::, but the path stays unchanged.
       path = [NSBezierPath bezierPathWithRect: aRect];
       [path transformUsingAffineTransform: ctm];
-      [self _setPath:YES];
+      [self _setPath];
       cairo_clip(_ct);
       cairo_paint(_ct);
       cairo_restore(_ct);
@@ -1353,8 +1296,8 @@ _set_op(cairo_t *ct, NSCompositingOperation op)
       srcRectInBase.origin.y -= 2 * (source->offset.y - ssize.height);
     }
 
-  x = floorf(destPointInBase.x);
-  y = floorf(destPointInBase.y + 0.5);
+  x = destPointInBase.x;
+  y = destPointInBase.y;
   minx = NSMinX(srcRectInBase);
   miny = NSMinY(srcRectInBase);
   width = NSWidth(srcRectInBase);
@@ -1408,7 +1351,7 @@ doesn't support to use the receiver cairo target as the source. */
            fraction: (float)delta
 {
   NSAffineTransformStruct tstruct =  [ctm transformStruct];
-  cairo_surface_t *src = cairo_get_target(source->_ct);
+  cairo_surface_t *src;
   double width, height;
   double x, y;
   cairo_pattern_t *cpattern;
@@ -1417,9 +1360,13 @@ doesn't support to use the receiver cairo target as the source. */
 
   if (!_ct || !source->_ct)
     {
+      NSLog(@"WARNING: -drawGState called with a NULL target context (%p) or source context (%p)",
+	    _ct, source->_ct);
       return;
     }
 
+  src = cairo_get_target(source->_ct);
+
   cairo_save(_ct);
 
   cairo_new_path(_ct);
@@ -1433,8 +1380,8 @@ doesn't support to use the receiver cairo target as the source. */
       aRect.origin.y -= 2*(source->offset.y - size.height);
     }
 
-  x = floorf(aPoint.x);
-  y = floorf(aPoint.y + 0.5);
+  x = aPoint.x;
+  y = aPoint.y;
   width = NSWidth(aRect);
   height = NSHeight(aRect);
 
@@ -1458,6 +1405,10 @@ doesn't support to use the receiver cairo target as the source. */
   cairo_matrix_init_scale(&source_matrix, 1, -1);
   cairo_matrix_translate(&source_matrix, 0, -[source->_surface size].height);
   cairo_pattern_set_matrix(cpattern, &source_matrix);
+  if (cairo_version() >= CAIRO_VERSION_ENCODE(1, 6, 0))
+    {
+      cairo_pattern_set_extend(cpattern, CAIRO_EXTEND_PAD);
+    }
   cairo_set_source(_ct, cpattern);
   cairo_pattern_destroy(cpattern);
   cairo_rectangle(_ct, aRect.origin.x, aRect.origin.y, width, height);
@@ -1482,13 +1433,16 @@ doesn't support to use the receiver cairo target as the source. */
 - (void *) saveClip
 {
 #if CAIRO_VERSION > CAIRO_VERSION_ENCODE(1, 4, 0)
-  cairo_status_t status;
-  cairo_rectangle_list_t *clip_rects = cairo_copy_clip_rectangle_list(_ct);
-
-  status = cairo_status(_ct);
-  if (status == CAIRO_STATUS_SUCCESS)
+  if (_ct)
     {
-      return clip_rects;
+      cairo_status_t status;
+      cairo_rectangle_list_t *clip_rects = cairo_copy_clip_rectangle_list(_ct);
+
+      status = cairo_status(_ct);
+      if (status == CAIRO_STATUS_SUCCESS)
+        {
+          return clip_rects;
+        }
     }
 #endif
 
@@ -1498,7 +1452,7 @@ doesn't support to use the receiver cairo target as the source. */
 - (void) restoreClip: (void *)savedClip
 {
 #if CAIRO_VERSION > CAIRO_VERSION_ENCODE(1, 4, 0)
-  if (savedClip)
+  if (_ct && savedClip)
     {
       int i;
       cairo_rectangle_list_t *clip_rects = (cairo_rectangle_list_t *)savedClip;
@@ -1547,38 +1501,49 @@ doesn't support to use the receiver cairo target as the source. */
                radius: (CGFloat)endRadius
               options: (NSUInteger)options
 {
-  int i;
-  int stops = [gradient numberOfColorStops];
-  NSPoint startP = [ctm transformPoint: startCenter];
-  NSPoint endP = [ctm transformPoint: endCenter];
-  cairo_pattern_t *cpattern = cairo_pattern_create_radial(startP.x, startP.y, 
-                                                          floatFromUserSpace(ctm, startRadius),
-                                                          endP.x, endP.y, 
-                                                          floatFromUserSpace(ctm, endRadius));
-  for (i = 0; i < stops; i++)
+  if (_ct)
     {
-      NSColor *color;
-      CGFloat location;
-      double red;
-      double green;
-      double blue;
-      double alpha;
+      cairo_status_t status;
+      int i;
+      int stops = [gradient numberOfColorStops];
+      NSPoint startP = [ctm transformPoint: startCenter];
+      NSPoint endP = [ctm transformPoint: endCenter];
+      cairo_pattern_t *cpattern = cairo_pattern_create_radial(startP.x, startP.y, 
+                                                              doubleFromUserSpace(ctm, startRadius),
+                                                              endP.x, endP.y, 
+                                                              doubleFromUserSpace(ctm, endRadius));
+      status = cairo_pattern_status(cpattern);
+      if (status != CAIRO_STATUS_SUCCESS)
+        {
+          return;
+        }
 
-      [gradient getColor: &color
-                location: &location
-                atIndex: i];
-      red = [color redComponent];
-      green = [color greenComponent];
-      blue = [color blueComponent];
-      alpha = [color alphaComponent];
-      cairo_pattern_add_color_stop_rgba(cpattern, location,
-                                        red, green, blue, alpha);
+      for (i = 0; i < stops; i++)
+        {
+          NSColor *color;
+          CGFloat location;
+          double red;
+          double green;
+          double blue;
+          double alpha;
+          
+          [gradient getColor: &color
+                    location: &location
+                     atIndex: i];
+          red = [color redComponent];
+          green = [color greenComponent];
+          blue = [color blueComponent];
+          alpha = [color alphaComponent];
+          cairo_pattern_add_color_stop_rgba(cpattern, location,
+                                            red, green, blue, alpha);
+        }
+      
+      cairo_save(_ct);
+      cairo_set_source(_ct, cpattern);
+      cairo_pattern_destroy(cpattern);
+      cairo_paint(_ct);
+      cairo_restore(_ct);
     }
-  cairo_save(_ct);
-  cairo_set_source(_ct, cpattern);
-  cairo_pattern_destroy(cpattern);
-  cairo_paint(_ct);
-  cairo_restore(_ct);
 }
 
 - (void) drawGradient: (NSGradient*)gradient
@@ -1586,37 +1551,48 @@ doesn't support to use the receiver cairo target as the source. */
               toPoint: (NSPoint)endPoint
               options: (NSUInteger)options
 {
-  int i;
-  int stops = [gradient numberOfColorStops];
-  NSPoint startP = [ctm transformPoint: startPoint];
-  NSPoint endP = [ctm transformPoint: endPoint];
-  cairo_pattern_t *cpattern = cairo_pattern_create_linear(startP.x, startP.y,
-                                                          endP.x, endP.y);
-
-  for (i = 0; i < stops; i++)
+  if (_ct)
     {
-      NSColor *color;
-      CGFloat location;
-      double red;
-      double green;
-      double blue;
-      double alpha;
+      cairo_status_t status;
+      int i;
+      int stops = [gradient numberOfColorStops];
+      NSPoint startP = [ctm transformPoint: startPoint];
+      NSPoint endP = [ctm transformPoint: endPoint];
+      cairo_pattern_t *cpattern = cairo_pattern_create_linear(startP.x, startP.y,
+                                                              endP.x, endP.y);
+      
+      status = cairo_pattern_status(cpattern);
+      if (status != CAIRO_STATUS_SUCCESS)
+        {
+          return;
+        }
 
-      [gradient getColor: &color
-                location: &location
-                atIndex: i];
-      red = [color redComponent];
-      green = [color greenComponent];
-      blue = [color blueComponent];
-      alpha = [color alphaComponent];
-      cairo_pattern_add_color_stop_rgba(cpattern, location,
-                                        red, green, blue, alpha);
+      for (i = 0; i < stops; i++)
+        {
+          NSColor *color;
+          CGFloat location;
+          double red;
+          double green;
+          double blue;
+          double alpha;
+          
+          [gradient getColor: &color
+                    location: &location
+                     atIndex: i];
+          red = [color redComponent];
+          green = [color greenComponent];
+          blue = [color blueComponent];
+          alpha = [color alphaComponent];
+          cairo_pattern_add_color_stop_rgba(cpattern, location,
+                                            red, green, blue, alpha);
+        }
+      
+      cairo_save(_ct);
+      cairo_set_source(_ct, cpattern);
+      cairo_pattern_destroy(cpattern);
+      cairo_paint(_ct);
+      cairo_restore(_ct);
     }
-  cairo_save(_ct);
-  cairo_set_source(_ct, cpattern);
-  cairo_pattern_destroy(cpattern);
-  cairo_paint(_ct);
-  cairo_restore(_ct);
 }
 
 @end
