@@ -318,12 +318,12 @@ static NSMapTable *gtable;
   [gstate GSSetStrokeColorspace: spaceref];
 }
 
-- (void) GSSetFillColor: (const float *)values
+- (void) GSSetFillColor: (const CGFloat *)values
 {
   [gstate GSSetFillColor: values];
 }
 
-- (void) GSSetStrokeColor: (const float *)values
+- (void) GSSetStrokeColor: (const CGFloat *)values
 {
   [gstate GSSetStrokeColor: values];
 }
@@ -427,6 +427,11 @@ static NSMapTable *gtable;
   [gstate GSShowGlyphs: glyphs : length];
 }
 
+- (void) GSShowGlyphsWithAdvances: (const NSGlyph *)glyphs : (const NSSize *)advances : (size_t) length
+{
+  [gstate GSShowGlyphsWithAdvances: glyphs : advances : length];
+}
+
 /* ----------------------------------------------------------------------- */
 /* Gstate Handling */
 /* ----------------------------------------------------------------------- */
