@@ -28,16 +28,15 @@
 
 #include "x11/XGServer.h"
 #include "x11/XGServerWindow.h"
-
 #include "x11/XWindowBuffer.h"
 
 #include <math.h>
 #include <sys/ipc.h>
 #include <sys/shm.h>
 
-
+#ifdef HAVE_XSHAPE
 #include <X11/extensions/shape.h>
-
+#endif
 
 static XWindowBuffer **window_buffers;
 static int num_window_buffers;
@@ -251,6 +250,7 @@ no_xshm:
                         wi->window->xframe.size.width, wi->window->xframe.size.height);*/
       if (wi->ximage)
         {
+#ifdef XSHM
           if (wi->use_shm)
             {
               XShmDetach(wi->display, &wi->shminfo);
@@ -258,6 +258,7 @@ no_xshm:
               shmdt(wi->shminfo.shmaddr);
             }
           else
+#endif
             XDestroyImage(wi->ximage);
         }
       if (wi->pixmap)
@@ -426,6 +427,7 @@ extern int XShmGetEventBase(Display *d);
 
 - (void) _gotShmCompletion
 {
+#ifdef XSHM
   if (!use_shm)
     return;
 
@@ -459,6 +461,7 @@ extern int XShmGetEventBase(Display *d);
         }
     }
 //        XFlush(window->display);
+#endif
 }
 
 - (void) _exposeRect: (NSRect)rect
@@ -506,9 +509,10 @@ accuracy, we do the test using int: s.
   if (w <= 0 || h <= 0)
     return;
 
+#ifdef XSHM
   if (use_shm)
     {
-
+#ifdef HAVE_XSHAPE
       /* HACK: lets try to use shaped windows to get some use out of
          destination alpha */
       if (has_alpha && use_shape_hack)
@@ -585,6 +589,7 @@ static int warn = 0;
               XFreePixmap(display, p);
             }
         }
+#endif // HAVE_XSHAPE
 
       if (pending_event)
         {
@@ -644,7 +649,9 @@ static int warn = 0;
           }
       }
     }
-  else if (ximage)
+  else
+#endif
+    if (ximage)
     {
       XPutImage(display, drawable, gc, ximage, x, y, x, y, w, h);
     }
@@ -707,6 +714,7 @@ static int warn = 0;
           pixmap=0;
         }
 
+#ifdef XSHM
       if (use_shm)
         {
           XShmDetach(display, &shminfo);
@@ -714,6 +722,7 @@ static int warn = 0;
           shmdt(shminfo.shmaddr);
         }
       else
+#endif
         XDestroyImage(ximage);
     }
   if (alpha)
