@@ -1296,8 +1296,10 @@ static Region emptyRegion;
   [path relativeMoveToPoint: NSMakePoint(width * scale.width, 0)];
 }
 
-- (void) GSShowGlyphs: (const NSGlyph *)glyphs : (size_t) length
+
+- (void) GSShowGlyphsWithAdvances: (const NSGlyph *)glyphs : (const NSSize *)advances : (size_t) length
 {
+  // FIXME: Currently advances is ignored
   int width;
   NSSize scale;
   XPoint xp;
@@ -1541,13 +1543,13 @@ static Region emptyRegion;
 
   if (pattern != nil)
     {
-      NSBezierPath *path;
+      NSBezierPath *rpath;
 
-      path = [[NSBezierPath alloc] init];
-      [path appendBezierPathWithRect: NSMakeRect(x, y, w, h)];
-      [path transformUsingAffineTransform: ctm];
-      [self fillPath: path withPattern: pattern];
-      RELEASE(path);
+      rpath = [[NSBezierPath alloc] init];
+      [rpath appendBezierPathWithRect: NSMakeRect(x, y, w, h)];
+      [rpath transformUsingAffineTransform: ctm];
+      [self fillPath: rpath withPattern: pattern];
+      RELEASE(rpath);
       return;
     }
 
