@@ -27,6 +27,8 @@
    Boston, MA 02110-1301, USA.
 */
 
+#include "config.h"
+
 #include <Foundation/Foundation.h>
 #include <Foundation/NSUserDefaults.h>
 #include <AppKit/NSPasteboard.h>
@@ -35,6 +37,9 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <x11/xdnd.h>
+#if HAVE_XFIXES
+#include <X11/extensions/Xfixes.h>
+#endif
 
 /*
  *	Non-predefined atoms that are used in the X selection mechanism
@@ -77,6 +82,7 @@ static char *atom_names[] = {
   "image/png",
   "image/svg",
   "application/rtf",
+  "text/richtext"
 };
 static Atom atoms[sizeof(atom_names)/sizeof(char*)];
 
@@ -119,6 +125,7 @@ static Atom atoms[sizeof(atom_names)/sizeof(char*)];
 #define XG_MIME_PNG       atoms[32]
 #define XG_MIME_SVG       atoms[33]
 #define XG_MIME_APP_RTF   atoms[34]
+#define XG_MIME_TEXT_RICHTEXT atoms[35]
 
 
 
@@ -165,6 +172,9 @@ static Atom atoms[sizeof(atom_names)/sizeof(char*)];
 - (void) xSelectionClear;
 - (void) xSelectionNotify: (XSelectionEvent*)xEvent;
 - (void) xSelectionRequest: (XSelectionRequestEvent*)xEvent;
+#if HAVE_XFIXES
++ (void) xFixesSelectionNotify: (XFixesSelectionNotifyEvent*)xEvent;
+#endif
 - (BOOL) xProvideSelection: (XSelectionRequestEvent*)xEvent;
 - (Time) xTimeByAppending;
 - (BOOL) xSendData: (unsigned char*) data format: (int) format 
@@ -191,6 +201,7 @@ static Window		xAppWin;
 static NSMapTable	*ownByX;
 static NSMapTable	*ownByO;
 static NSString		*xWaitMode = @"XPasteboardWaitMode";
+static int              xFixesEventBase;
 
 @implementation	XPbOwner
 
@@ -254,6 +265,32 @@ static NSString		*xWaitMode = @"XPasteboardWaitMode";
       forMode: xWaitMode];
 
   XSelectInput(xDisplay, xAppWin, PropertyChangeMask);
+
+#if HAVE_XFIXES
+  {
+    int error;
+
+    // Subscribe to notifications of when the X clipboard changes,
+    // so we can invalidate our cached list of types on it.
+    //
+    // FIXME: If we don't have Xfixes, we should really set up a polling timer.
+    
+    if (XFixesQueryExtension(xDisplay, &xFixesEventBase, &error))
+      {
+       XFixesSelectSelectionInput(xDisplay, xAppWin, XA_CLIPBOARD,
+                                  XFixesSetSelectionOwnerNotifyMask |
+                                  XFixesSelectionWindowDestroyNotifyMask |
+                                  XFixesSelectionClientCloseNotifyMask );
+       XFixesSelectSelectionInput(xDisplay, xAppWin, XA_PRIMARY,
+                                  XFixesSetSelectionOwnerNotifyMask |
+                                  XFixesSelectionWindowDestroyNotifyMask |
+                                  XFixesSelectionClientCloseNotifyMask);
+       // FIXME: Also handle the dnd pasteboard
+       NSDebugLLog(@"Pbs", @"Subscribed to XFixes notifications");
+      }
+  }
+#endif
+
   XFlush(xDisplay);
 
   /*
@@ -379,6 +416,14 @@ static NSString		*xWaitMode = @"XPasteboardWaitMode";
       break;
 
     default:
+#if HAVE_XFIXES
+      if (xEvent->type == xFixesEventBase + XFixesSelectionNotify)
+       {
+         [self xFixesSelectionNotify: (XFixesSelectionNotifyEvent*)xEvent];
+         break;
+       }
+#endif
+
       NSDebugLLog(@"Pbs", @"Unexpected X event.");
       break;
     }
@@ -473,6 +518,27 @@ static NSString		*xWaitMode = @"XPasteboardWaitMode";
   [o xSelectionNotify: xEvent];
 }
 
+#if HAVE_XFIXES
++ (void) xFixesSelectionNotify: (XFixesSelectionNotifyEvent*)xEvent
+{
+  XPbOwner *o = [self ownerByXPb: xEvent->selection];
+  
+  if (o != nil)
+    {
+      if (xEvent->owner != (Window)xAppWin)
+       {
+         NSDebugLLog(@"Pbs", @"Notified that selection %@ changed", [[o osPb] name]);
+	 // FIXME: Invalidate the cached types in the pasteboard since they are no longer valid
+       }
+      else
+       {
+	 // The notification is telling us that we became the selection owner,
+	 // which we already know about since we must have initiated that change.
+       }
+    }
+}
+#endif
+
 + (void) xSelectionRequest: (XSelectionRequestEvent*)xEvent
 {
   XPbOwner		*o;
@@ -551,6 +617,14 @@ static NSString		*xWaitMode = @"XPasteboardWaitMode";
    *	so we must tell the X server that we have the current selection.
    *	To conform to ICCCM we need to specify an up-to-date timestamp.
    */
+
+  // FIXME: See note in -xSelectionClear:. This method is called by
+  // -[NSPasteboard declareTypes:owner:], but we might not want
+  // GNUstep to take ownership. (e.g. suppose selection ownership changes from
+  // gnome-terminal to OpenOffice.org. But, we will still need to update the
+  // types available on the pasteboard in case a GNUstep app wants to read from
+  // the pasteboard.)
+
   _timeOfSetSelectionOwner = [self xTimeByAppending];
   XSetSelectionOwner(xDisplay, _xPb, xAppWin, _timeOfSetSelectionOwner);
 
@@ -651,6 +725,8 @@ static NSString		*xWaitMode = @"XPasteboardWaitMode";
       [self requestData: XG_MIME_RTF];
       if ([self data] == nil)
         [self requestData: XG_MIME_APP_RTF];
+      if ([self data] == nil)
+        [self requestData: XG_MIME_TEXT_RICHTEXT];
     }
   else if ([type isEqual: NSTIFFPboardType])
     {
@@ -720,6 +796,8 @@ xErrorHandler(Display *d, XErrorEvent *e)
   unsigned int i;
   Atom *targets;
 
+  NSDebugLLog(@"Pbs", @"%@ availableTypes called", [[self osPb] name]);
+
   [self setData: nil];
   [self requestData: XG_TARGETS];
   data = [self data];
@@ -729,6 +807,9 @@ xErrorHandler(Display *d, XErrorEvent *e)
   count = [data length] / sizeof(Atom);
   targets = (Atom*)[data bytes];
   types = [NSMutableArray arrayWithCapacity: count];
+
+  NSDebugLLog(@"Pbs", @"%@ availableTypes: %d types available", [[self osPb] name], count);
+
   for (i = 0; i < count; i++)
     {
       Atom type;
@@ -747,7 +828,8 @@ xErrorHandler(Display *d, XErrorEvent *e)
           [types addObject: NSFilenamesPboardType];
         }
       else if ((type == XG_MIME_RTF)
-               || (type == XG_MIME_APP_RTF))
+               || (type == XG_MIME_APP_RTF)
+	       || (type == XG_MIME_TEXT_RICHTEXT))
         {
           [types addObject: NSRTFPboardType];
         }
@@ -767,7 +849,7 @@ xErrorHandler(Display *d, XErrorEvent *e)
         }
       // FIXME: Support more types
       else
-        {
+	{
           char *name = XGetAtomName(xDisplay, type);
 
           // FIXME: We should rather add this type to the
@@ -775,7 +857,7 @@ xErrorHandler(Display *d, XErrorEvent *e)
           NSDebugLLog(@"Pbs", @"Unsupported selection type '%s' available", 
                       name);
           XFree(name);
-        }
+	}
     }
 
   return types;
@@ -788,6 +870,8 @@ xErrorHandler(Display *d, XErrorEvent *e)
  */
 - (void) xSelectionClear
 {
+  // FIXME: This will cause -pasteboardChangedOwner: to be called, which will
+  // take ownership of the X selection. That is probably wrong...
   [_pb declareTypes: [self availableTypes] owner: self];
   [self setOwnedByOpenStep: NO];
 }
@@ -816,7 +900,7 @@ xErrorHandler(Display *d, XErrorEvent *e)
                                   xEvent->property,
                                   long_offset,         // offset
                                   long_length,
-                                  True,               // Delete prop when read.
+                                  False,               // Aug 2011 - changed to False (don't delete property)
                                   req_type,
                                   &actual_type,
                                   &actual_format,
@@ -826,7 +910,17 @@ xErrorHandler(Display *d, XErrorEvent *e)
       
       if ((status == Success) && (number_items > 0))
         {
-          long count = number_items * actual_format / 8;
+          long count;
+	  if (actual_type == XA_ATOM)
+	    {
+	      // xlib will report an actual_format of 32, even if
+	      // data contains an array of 64-bit Atoms
+	      count = number_items * sizeof(Atom);
+	    }
+	  else
+	    {
+	      count = number_items * actual_format / 8;
+	    }
             
           if (md == nil)
             {
@@ -985,7 +1079,8 @@ xErrorHandler(Display *d, XErrorEvent *e)
           RELEASE(url);
         }
       else if ((actual_type == XG_MIME_RTF)
-               || (actual_type == XG_MIME_APP_RTF))
+               || (actual_type == XG_MIME_APP_RTF)
+	       || (actual_type == XG_MIME_TEXT_RICHTEXT))
         {
           [self setData: md];
         }
@@ -1093,6 +1188,7 @@ xErrorHandler(Display *d, XErrorEvent *e)
         {
           xTypes[numTypes++] = XG_MIME_RTF;
           xTypes[numTypes++] = XG_MIME_APP_RTF;
+	  xTypes[numTypes++] = XG_MIME_TEXT_RICHTEXT;
         }
 
       if ([types containsObject: NSTIFFPboardType])
@@ -1319,7 +1415,8 @@ xErrorHandler(Display *d, XErrorEvent *e)
         }
     }
   else if (((xEvent->target == XG_MIME_RTF) 
-            || (xEvent->target == XG_MIME_APP_RTF))
+            || (xEvent->target == XG_MIME_APP_RTF)
+	    || (xEvent->target == XG_MIME_TEXT_RICHTEXT))
            && [types containsObject: NSRTFPboardType])
     {
       NSData *d = [_pb dataForType: NSRTFPboardType];
